package Pinto::Store::Git;

# ABSTRACT: Store your Pinto repository with Git

use Moose;

extends 'Pinto::Store';

#-------------------------------------------------------------------------------

our $VERSION = '0.005'; # VERSION

#-------------------------------------------------------------------------------

1;



=pod

=for :stopwords Jeffrey Ryan Thalhammer Imaginative Software Systems

=head1 NAME

Pinto::Store::Git - Store your Pinto repository with Git

=head1 VERSION

version 0.005

=head1 DESCRIPTION

This module is Not yet implimented.

=head1 AUTHOR

Jeffrey Ryan Thalhammer <jeff@imaginative-software.com>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2011 by Imaginative Software Systems.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut


__END__

